﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace RecrownedAthenaeum.Render
{
    /// <summary>
    /// Box that crops off anything outside of the bounds.
    /// </summary>
    public class ScissorBox
    {

        SpriteBatch spriteBatch;
        Rectangle currentScissorRect;

        /// <summary>
        /// Starts spritebatch with scissoring enabled.
        /// </summary>
        /// <param name="rectangle"></param>
        /// <param name="spriteBatch"></param>
        /// <param name="spriteBatchSettings">The settings for using the <see cref="SpriteBatch"/> to perform the scissor.</param>
        public void Begin(Rectangle rectangle, SpriteBatch spriteBatch, SpriteBatchSettings? spriteBatchSettings = null)
        {
            if (spriteBatchSettings == null) spriteBatchSettings = Configuration.SpriteBatchSettings;

            this.spriteBatch = spriteBatch;
            spriteBatchSettings.Value.rasterizerState.ScissorTestEnable = true;
            spriteBatchSettings.Value.BeginSpriteBatch(spriteBatch);
            currentScissorRect = spriteBatch.GraphicsDevice.ScissorRectangle;
            spriteBatch.GraphicsDevice.ScissorRectangle = rectangle;
        }

        /// <summary>
        /// Ends the scissoring and spritebatch.
        /// </summary>
        public void End()
        {
            spriteBatch.GraphicsDevice.ScissorRectangle = currentScissorRect;
            spriteBatch.End();
        }
    }
}
